/*
 * Boo Development Tools for the Eclipse IDE
 * Copyright (C) 2005 Rodrigo B. de Oliveira (rbo@acm.org)
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package aQute.bnd.plugin.editors;

import java.util.*;

import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.*;
import org.eclipse.jface.text.source.ISharedTextColors;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.RGB;

import aQute.lib.osgi.*;

public class BndScanner extends RuleBasedScanner {
	static RGB	C_COMMENT		= new RGB(128, 0, 0);
	static RGB	C_INSTRUCTION	= new RGB(158, 158, 158);
	static RGB	C_OPTION		= new RGB(0, 128, 255);
	static RGB	C_DEFAULT		= new RGB(0, 0, 0);
	static RGB	C_DIRECTIVE		= new RGB(255, 255, 0);
	static RGB	C_PROPERTY		= new RGB(255, 255, 0);
	static RGB  C_COMPONENT     = new RGB(255,255,0);
	
	static Token createBoldToken(ISharedTextColors manager, RGB rgb) {
		return new Token(new TextAttribute(manager.getColor(rgb), null,
				SWT.BOLD));
	}

	static Token createToken(ISharedTextColors manager, RGB rgb) {
		return new Token(new TextAttribute(manager.getColor(rgb)));
	}
	static class BndWordRule implements IRule {

		private Map				_words	= new HashMap();
		private StringBuffer	_buffer	= new StringBuffer();
		private IToken			other;

		public BndWordRule(ISharedTextColors manager) {
			other = createToken(manager, C_DEFAULT);
			addWords( new String[] {"abc", "def"}, createBoldToken(manager,C_PROPERTY));
			addWords(Analyzer.headers, createBoldToken(manager, C_INSTRUCTION));
			addWords(Analyzer.options, createBoldToken(manager, C_OPTION));
			addWords(Analyzer.directives, createBoldToken(manager, C_DIRECTIVE));
			addWords(Analyzer.componentDirectives, createBoldToken(manager, C_COMPONENT));
		}


		private boolean isWordStart(char c) {
			return Character.isJavaIdentifierStart(c);
		}

		private boolean isWordPart(char c) {
			return Character.isJavaIdentifierPart(c) || c=='-';
		}

		public IToken evaluate(ICharacterScanner scanner) {
			int c = scanner.read();
			if (isWordStart((char) c)) {
				_buffer.setLength(0);
				do {
					_buffer.append((char) c);
					c = scanner.read();
				} while (c != ICharacterScanner.EOF && isWordPart((char) c));
				scanner.unread();

				IToken token = (IToken) _words.get(_buffer.toString());
				if (token != null)
					return token;
				return other;
			}

			scanner.unread();
			return Token.UNDEFINED;

		}

		private int peekNextValidChar(ICharacterScanner scanner) {
			int readCount = 1;
			int c = scanner.read();
			while (c != ICharacterScanner.EOF &&
					Character.isWhitespace((char) c)) {
				c = scanner.read();
				++readCount;
			}
			while (readCount > 0) {
				scanner.unread();
				--readCount;
			}
			return c;
		}

		private void addWords(String[] words, IToken token) {
			for (int i = 0; i < words.length; ++i) {
				_words.put(words[i], token);
			}
		}

	}

	public BndScanner(ISharedTextColors manager) {
		IToken defaultToken = new Token(new TextAttribute(manager
				.getColor(C_DEFAULT)));

		IRule[] rules = new IRule[] {
				new WhitespaceRule(new IWhitespaceDetector() {
					public boolean isWhitespace(char c) {
						return (c == ' ' || c == '\t' || c == '\n' || c == '\r');
					}
				}), new BndWordRule(manager)};

		setRules(rules);
		setDefaultReturnToken(defaultToken);
	}
}
